/'--------------------------------------------------------------------
                  Nascent World Engine for FreeBASIC              
                     Copyright Vincent DeCampo 2009      

   This program is free software: you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------'/
'
#Include Once "fbgfx.bi"
#Include Once "file.bi"
#Include Once "crt.bi"
#Include Once "tmsupport.bi"
#Include Once "png.bi"
#Include Once "sound.bi"

#Define EngineVersion   "0.3.4"

#Define DisplayWidth    600
#Define DisplayHeight   368

#Define TileWidth	 		32
#Define TileHeight 		32 
#Define TileWidth_2		TileWidth\2
#Define TileHeight_2		TileHeight\2 

#Define DefaultSpeed    .1
#Define EngineFPS       33

#Define ScreenToTileX(x) (x)\TileWidth
#Define ScreenToTileY(y) (y)\TileHeight
#Define TileXToScreen(x) (x)*TileWidth
#Define TileYToScreen(y) (y)*TileHeight

#Define MaxSpriteIdx		 200
#Define MaxObjectIdx     1000
#Define MaxAnimationIdx  10
#Define MaxSheetIdx      10
#Define MaxLightMapIdx   20
#Define MaxLightDistance 3

#Macro DeAlloc(x)
   DeAllocate(x)
   x=0   
#EndMacro

#Define bNOT(x)   IIf((x)=0,1,0)

Type _CustomLayer
   '
   Declare Property LoadLayer (filename As String)
   As Single        speed
   As UInteger      Color
   As Integer       alpha = 0
   As Any Ptr       image
   As Single        position
   '      
End Type

Type _MapCell Field=1
   As Short    Frame
   As Short    Sheet
   As Integer  Attr
   As Integer  Animation
   As Integer  Reserved2
End Type

Type _Animation
   '
   Declare Constructor (index As Integer, fStart As Integer, fStop As Integer, FPS As Integer, floop As Integer=0)
   Declare Function GetFrame    () As Integer
   Declare Function GetNextFrame() As Integer
   Declare Function GetPrevFrame() As Integer
   Declare Sub      Reset       () 
   '
   As Integer SheetIndex
   As Integer Pause
   '
Private:
   '
   As Integer StartFrame
   As Integer StopFrame
   As Integer CurrentFrame
   As Double  FrameTick
   As Double  NextFrame
   As Integer FrameLoop
   '
End Type

Type _GameClock
   '
   Declare Constructor (FPS As Integer)
   Declare Function TickTime() As Integer
   '
   As Integer FPS
   '
Private:
   '
   As Double  FrameTick
   As Double  NextFrame
   As Double  FPSCounter
   As Integer fcount
      
End Type

Type _Object
   '
   Declare Destructor()
   '
	Declare Sub MoveLeft	(Speed As Single = DefaultSpeed)
	Declare Sub MoveRight(Speed As Single = DefaultSpeed)
	Declare Sub MoveUp	(Speed As Single = DefaultSpeed)
	Declare Sub MoveDown	(Speed As Single = DefaultSpeed)
	Declare Sub SetObject(x As Single, y As Single)
	'
	Declare Function DistanceTo  (Object As _Object) As Double
	Declare Function CollidedWith(Object As _Object) As Integer
	'
	As Integer     active  = 0
	As Integer     objType = 0
	As Single	   x
	As Single   	y
	As Integer     dx
	As Integer     dy
	As Single      lastx
	As Single      lasty
	As Integer     imgIdx  = -1
	As Integer     rotangle
	' 
   As _Animation Ptr Animation	
   '
   As Any Ptr     LastImg
   '    
End Type

Type _Map
	'
   'Map Effects
	Enum
      mapANIMATE     = &h1
      mapINVISIBLE   = &h2
      mapNONBLOCKING = &h4
	End Enum
	'
	Declare Destructor ()
	Declare Function LoadMap  (filename As String, mWidth As Integer, mHeight As Integer) As Integer
	Declare Function MakeMap  (mWidth As Integer, mHeight As Integer) As Integer
	Declare Function SaveMap  (filename As String) As Integer
	Declare Function GetCell  (TileX As Integer, TileY As Integer) As _MapCell
	Declare Sub		  SetCell  (TileX As Integer, TileY As Integer, value As _MapCell) 
	Declare Sub      FillMap  (frame As Integer=-1, sheet As Integer=-1, attr As Integer = -1) 
   Declare Function Collision(Object As _Object) As Integer
	Declare Function BuildCell(Frame As Short, Sheet As Short, TileAttr As Integer, Animation As Integer) As _MapCell

	'
	As Integer BoundaryStyle   = -1
	As Integer BoundaryGrid    = 1
	'
	As _Animation Ptr Animation(MaxAnimationIdx)	
	'
	As _Object Ptr lightsource
   '
Private:
	As Integer MapWidth
	As Integer MapHeight
	As Integer MaxOffset
	As _MapCell Ptr MAP
   '
End Type

Type _Camera
   '   
	Declare Sub MoveLeft	      (Speed As Single = DefaultSpeed)
	Declare Sub MoveRight      (Speed As Single = DefaultSpeed)
	Declare Sub MoveUp	      (Speed As Single = DefaultSpeed)
	Declare Sub MoveDown	      (Speed As Single = DefaultSpeed)
	Declare Sub CameraSet      (x As Single, y As Single)
	Declare Sub CenterOn       (object As _Object)
	As Single	x
	As Single	y
   As Integer  dspWidth
   As Integer  dspHeight	
	'
End Type

Type _SpriteSheet
   As Integer        Cols
   As Integer        Rows
   As Integer        SpriteWidth
   As Integer        SpriteHeight
   As Any Ptr        SpriteSheet	
   As Integer        SheetWidth
   As Integer        SheetHeight   
End Type

Type _Sprites
	Declare Constructor ()
	Declare Destructor  ()
	Declare Function LoadSprite (index As Integer, filename As String) As Any Ptr
	Declare Function GetImage 	 (index As Integer) As Any Ptr
   Declare Function LoadSheet  (index As Integer, filename As String, TileDx As Integer, TileDy As Integer) As Integer
   Declare Sub      SheetPut   (dst As Any Ptr, index As Integer, x As Integer, y As Integer, fnum As Integer, trans As Integer = 1) 
   '
   As _SpriteSheet   Sheet(MaxSheetIdx)
   '
Private:
   '
	As Integer Ptr	   imgStore
	As Integer 		   imgCount
	'
End Type

Type _tmEngine
   '   
   Declare Constructor()   
   Declare Constructor(dispWidth As Integer, dispHeight As Integer)
   Declare Destructor()
   '
	Declare Function RenderAll        () As Integer
	Declare Function RenderMap        () As Integer
	Declare Function RenderMask       () As Integer
	Declare Function RenderObjects    () As Integer
	Declare Function RenderFog        () As Integer
	Declare Function RenderLayer      (layer As _CustomLayer) As Integer
	Declare Function RenderSceneAt (x As Integer, y As Integer, zoom As Single=1.0f) As Integer
	Declare Function IsOnScreen    (x As Single, y As Single) As Integer
	Declare Function GetMouseTile  (ByRef x As Integer, ByRef y As Integer) As Integer
	Declare Function HighlightTile (x As Integer, y As Integer) As Integer
	Declare Sub      ShowMouse     ()
   Declare Property SetFog        (fogval As Integer) 
   Declare Property EnableMask    (enable As Integer)
	'
	As _Map 		      Map
	As _Camera 	      Camera
	As _Object        Object(MaxObjectIdx)
	As _Sprites       Sprites
	As _GameClock     GameClock = _GameClock(EngineFPS)
	'
	As Integer        DesignMode
Private:
   As Integer        dspWidth
   As Integer        dspHeight
   As Integer        TilesPerRow
   As Integer        TilesPerCol
   '
	As Integer        Fog       = 0
   As Any Ptr        foglayer	
   '
   As Any Ptr        RenderSurface
   As Any Ptr        ZoomSurface
   As Integer        RenderOffsetX
   As Integer        RenderOffsetY   
   '
   As Any Ptr        workimg1
   As Any Ptr        workimg2
   As Any Ptr        mask
   '
End Type

#Ifndef _MAKELIB_
   #Inclib  "nascentwe"
#EndIf
