/'--------------------------------------------------------------------
                  Nascent World Engine for FreeBASIC              
                     Copyright Vincent DeCampo 2009      

   this program is free software: you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

   this program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

--------------------------------------------------------------------'/

'-------------------------------------------------------
'                 		LoadBitmap
'-------------------------------------------------------
Function LoadBitmap (filename As String) As Any Ptr
Dim As String *2 head
Dim As Integer h,w,hf
Dim As Any Ptr img

   If FileExists(filename) Then

      hf=FreeFile
      Open filename For Binary As #hf
         Get #hf,,head

         If head = "BM" Then 'BM header

            Seek #hf,&h13
            Get  #hf,,w   'get width
            Seek #hf,&h17
            Get  #hf,,h   'get height
				
				Close #hf
	
	         img = ImageCreate(w,h)
      		BLoad filename,img
      		           
         End If            

      Close #hf 
       
   EndIf

   Return img           
      
End Function

'-------------------------------------------------------
'                 		Game Math
'-------------------------------------------------------

Function _tmGameMath.DegToRad(Deg As Single) As Single

    Return (PI * (Deg / 180))

End Function

Function _tmGameMath.RadToDeg(Rad As Single) As Single

    Return (180 * (Rad / PI))

End Function

Function _tmGameMath.Distance(ByVal x1 As Single,ByVal y1 As Single,ByVal  x2 As Single,ByVal y2 As Single) As Single

    Return ( Sqr((x1 - x2) ^ 2 + (y1 - y2) ^ 2) )

End Function

Function _tmGameMath.GetAngle(OrgX As Single, OrgY As Single, x2 As Single, y2 As Single) As Single
Dim As Single x,y

   x = x2-OrgX
   y = y2-Orgy
   
   Return Atn2(x,y)
    
End Function

Function _tmGameMath.GetAngleN(OrgX As Single, OrgY As Single, x2 As Single, y2 As Single) As Single
Dim As Single x,y

   x = x2-OrgX
   y = y2-Orgy
   
   Return NormalAng(Atn2(x,y))
    
End Function

Function _tmGameMath.NormalAng (Ang As Single) As Single
   If Ang > 0 Then
      Return (PI*2)-Ang
   Else
      Return Abs(Ang)
   EndIf
End Function

Sub _tmGameMath.RotateCoords(X As Single, Y As Single, degRotation As Single, ByRef NewX As Single, ByRef NewY As Single)
Dim Radius As Single
Dim Angle As Single
Dim NewAngle As Single
Dim RotAngle As Single
	
	Radius   = Distance(0, 0, X, Y)
	Angle    = GetAngle(0, 0, X, Y)
	RotAngle = DegToRad (degRotation)
	NewAngle = (Angle + RotAngle)	
       
	NewX = Cos(NewAngle) * Radius
	NewY = Sin(NewAngle) * Radius
                  
End Sub

Function _tmGameMath.PolarX (ByVal Deg As Integer, ByVal Radius As Single, ByVal WorldWidth As Integer) As Single
	
	Return (Cos(DegToRad(Deg)) * Radius) + (WorldWidth/2)
	
End Function

Function _tmGameMath.PolarY (ByVal Deg As Integer, ByVal Radius As Single, ByVal WorldHeight As Integer) As Single
	
	Return (Sin(DegToRad(Deg)) * Radius) + (WorldHeight/2) 
	
End Function

'-------------------------------------------------------
'		                  Collider
'-------------------------------------------------------
'
Constructor _tmCollider (MaxWidth As Integer, MaxHeight As Integer, BPP As Integer)
	
	this.Container = ImageCreate (MaxWidth, MaxHeight, RGBA(255,0,255,255), BPP)
	this.BPP 		= BPP
	this.MXWidth 	= MaxWidth
	this.MXHeight 	= MaxHeight
		
	If this.Container = 0 Then
		Print "_tmCollider:Init Failed! GFX mode not set."
	EndIf
	
End Constructor

Function _tmCollider.ComparePixels32(Image1 As Any Ptr, Image2 As Any Ptr) As Integer
Dim FBimg1   As FB.Image Ptr = Image1
Dim FBimg2   As FB.Image Ptr = Image2
Dim As Integer Buff1, Buff2, BuffSize, Collide
Dim As Byte Ptr pPixel1, pPixel2

	Buff1 = FBimg1[0].width * FBimg1[0].height
	Buff2 = FBimg2[0].width * FBimg2[0].height
	BuffSize = IIf(Buff1>Buff2,Buff2,Buff1)
	pPixel1 = Image1 + SizeOf(FB.Image)
	pPixel2 = Image2 + SizeOf(FB.Image)
	
   Asm
   
   .intel_syntax 

      mov  ecx, Dword [BuffSize]
	
.balign 16

Start32:	
	
   	Sub     ecx, 1   
	   'Load up pixels in primary image   
	   'Load up pixels in second image
   	mov     eax, [pPixel1]	
   	mov     edx, [pPixel2]
   	mov     eax, [eax+ecx*4]
   	mov     edx, [edx+ecx*4]
   	Or      edx, &hFF000000
   	
   	'Check Primary Pixel Transparency
   	cmp     eax, 0xFFFF00FF
   	je      Ignore32
   
   	'Check Pixel Equality
   	cmp     eax, edx
   	je		  Ignore32
   
   	'Must be a collision
   	mov Dword Ptr [Collide], 1
   	jmp Done32

Ignore32:

	   jecxz  Done32
	   jmp    Start32

Done32:

   End Asm

	Return (Collide)
	
End Function

Function _tmCollider.ComparePixels16(Image1 As Any Ptr, Image2 As Any Ptr) As Integer
Dim FBimg1   As FB.Image Ptr = Image1
Dim FBimg2   As FB.Image Ptr = Image2
Dim As Integer Buff1, Buff2, BuffSize, Collide
Dim As Byte Ptr pPixel1, pPixel2

	Buff1 = FBimg1[0].width * FBimg1[0].height
	Buff2 = FBimg2[0].width * FBimg2[0].height
	BuffSize = IIf(Buff1>Buff2,Buff2,Buff1)
	pPixel1 = Image1 + SizeOf(FB.Image)
	pPixel2 = Image2 + SizeOf(FB.Image)
	
   Asm
   
   .intel_syntax
   
   	mov ecx, Dword [BuffSize]
   		
   .balign 16

Start16:	
   	
   	Sub ecx, 1   
	   
	   mov eax, Dword [pPixel1]
   	mov ebx, Dword [pPixel2]
   	mov  ax, Short [eax+ecx*2]
   	mov  bx, Short [ebx+ecx*2]
   	
   	cmp  ax, 0xF81F
   	je   Ignore16
   	
   	cmp  ax, bx
   	je   Ignore16
   
   	mov Dword Ptr [Collide], 1
   	jmp Done16
   
Ignore16:
   		
   	jecxz  Done16
   	jmp    Start16
   
Done16:
   	
   End Asm

	Return (Collide)

End Function

Function _tmCollider.Collision(Sprite1 As Any Ptr, X1 As Integer, Y1 As Integer, Sprite2 As Any Ptr, X2 As Integer, Y2 As Integer) As Integer
Dim FBimg1   As FB.Image Ptr = Sprite1
Dim FBimg2   As FB.Image Ptr = Sprite2
	
	If this.Container = 0 Then Return (-3)	'Object not initalized
	If Sprite1 			= 0 Then Return (-1)	'Sprite 1 not initalized
	If Sprite2 			= 0 Then Return (-2)	'Sprite 2 not initalized

	If (x1 + FBimg1[0].width < x2)   Then Return (0)   'Sprite completely left of other
	If (y1 + FBimg1[0].height < y2)  Then Return (0)   'Sprite completely above other
	If (x1 > x2 + FBimg2[0].width)   Then Return (0)   'Sprite completely right of other
	If (y1 > y2 + FBimg2[0].height)  Then Return (0)   'Sprite completely below other

	Line this.Container,(0,0)-(this.MXWidth,this.MXHeight),RGB(255,0,255),BF	'Clear buffer with Trans color
	Put  this.Container,(0,0),Sprite1,TRANS					'Put Sprite1 in buffer
	Put  this.Container,(X2-X1,Y2-Y1),Sprite2,TRANS			'Put Sprite2 in buffer
	
	Select Case BPP
	   Case 32
			Return ComparePixels32 (Sprite1,this.Container) 
	   Case 16
			Return ComparePixels16 (Sprite1,this.Container)
	End Select
	
End Function

Destructor _tmCollider
	ImageDestroy (Container)
End Destructor

'-------------------------------------------------------------------
'                    		Rotate Sprite
'-------------------------------------------------------------------
'Rotozoom for 32-bit FB.Image by Dr_D(Dave Stanley) and yetifoot(Simon Nash)
'No warranty implied... use at your own risk ;) 
'-------------------------------------------------------------------   
'
Sub RotoZoom(ByRef dst As FB.IMAGE Ptr = 0, ByRef src As Const FB.IMAGE Ptr, ByVal positx As Integer, ByVal posity As Integer, ByRef angle As Integer, ByRef zoomx As Single, ByRef zoomy As Single = 0, ByVal transcol As UInteger  = &h00ff00ff)
Static As Integer mx, my, col, nx, ny
Static As Single nxtc, nxts, nytc, nyts
Static As Single tcdzx, tcdzy, tsdzx, tsdzy
Static As Integer sw2, sh2, dw, dh
Static As Single tc, ts, _mx, _my
Static As UInteger Ptr dstptr, srcptr, odstptr
Static As Integer xput, yput, startx, endx, starty, endy
Static As Integer x(3), y(3), xa, xb, ya, yb, lx, ly
Static As UByte Ptr srcbyteptr, dstbyteptr
Static As Integer dstpitch, srcpitch, srcbpp, dstbpp, srcwidth, srcheight
    
   If zoomx = 0 Then Exit Sub
   If zoomy = 0 Then zoomy = zoomx
   
   If dst = 0 Then
      dstptr = ScreenPtr
      odstptr = dstptr
      ScreenInfo dw,dh,,,dstpitch
   Else
      dstptr = Cast( UInteger Ptr, dst + 1 )
      odstptr = Cast( UInteger Ptr, dst + 1 )
      dw = dst->Width
      dh = dst->height
      dstbpp = dst->bpp
      dstpitch = dst->pitch
   End If
   
   srcptr = Cast( UInteger Ptr, src + 1 )
   srcbyteptr = Cast( UByte Ptr, srcptr )
   dstbyteptr = Cast( UByte Ptr, dstptr )
   
   sw2 = src->Width\2
   sh2 = src->height\2
   srcbpp = src->bpp
   srcpitch = src->pitch
   srcwidth = src->Width
   srcheight = src->height
   
   tc = Cos( angle * pi_180 )
   ts = Sin( angle * pi_180 )
   tcdzx = tc/zoomx
   tcdzy = tc/zoomy
   tsdzx = ts/zoomx
   tsdzy = ts/zoomy
   
   xa = sw2 * tc * zoomx + sh2  * ts * zoomx
   ya = sh2 * tc * zoomy - sw2  * ts * zoomy
   
   xb = sh2 * ts * zoomx - sw2  * tc * zoomx
   yb = sw2 * ts * zoomy + sh2  * tc * zoomy
   
   x(0) = sw2-xa
   x(1) = sw2+xa
   x(2) = sw2-xb
   x(3) = sw2+xb
   y(0) = sh2-ya
   y(1) = sh2+ya
   y(2) = sh2-yb
   y(3) = sh2+yb
   
   For i As Integer = 0 To 3
     For j As Integer = i To 3
         If x(i)>=x(j) Then
            swap x(i), x(j)
         End If
     Next
   Next
   startx = x(0)
   endx = x(3)
   
   For i As Integer = 0 To 3
     For j As Integer = i To 3
         If y(i)>=y(j) Then
            swap y(i), y(j)
         End If
     Next
   Next
   
   starty = y(0)
   endy = y(3)
   
   positx-=sw2
   posity-=sh2
   
   If posity+starty<0 Then starty = -posity
   If positx+startx<0 Then startx = -positx
   If posity+endy<0 Then endy = -posity
   If positx+endx<0 Then endx = -positx
   
   If positx+startx>(dw-1) Then startx = (dw-1)-positx
   If posity+starty>(dh-1) Then starty = (dh-1)-posity
   If positx+endx>(dw-1) Then endx = (dw-1)-positx
   If posity+endy>(dh-1) Then endy = (dh-1)-posity
   If startx = endx Or starty = endy Then Exit Sub 
   
   xput = (startx + positx) * 4
   yput = starty + posity
   ny = starty - sh2
   nx = startx - sw2
   nxtc = (nx * tcdzx)
   nxts = (nx * tsdzx)
   nytc = (ny * tcdzy)
   nyts = (ny * tsdzy)
   dstptr += dstpitch * yput \ 4
   
	Dim As Integer y_draw_len = (endy - starty) + 1
	Dim As Integer x_draw_len = (endx - startx) + 1
    
   'and we're off!
   Asm
      mov edx, Dword Ptr [y_draw_len]
      
      Test edx, edx ' 0?
      jz y_end      ' nothing to do here
      
      fld Dword Ptr[tcdzy]
      fld Dword Ptr[tsdzy]
      fld Dword Ptr [tcdzx]
      fld Dword Ptr [tsdzx]
      
      y_inner:
      
      fld Dword Ptr[nxtc]     'st(0) = nxtc, st(1) = tsdzx, st(2) = tcdzx, st(3) = tsdzy, st(4) = tcdzy
      fsub Dword Ptr[nyts]    'nxtc-nyts
      fiadd Dword Ptr[sw2]    'nxtc-nyts+sw2
      
      fld Dword Ptr[nxts]     'st(0) = nxts, st(1) = tsdzx, st(2) = tcdzx, st(3) = tsdzy, st(4) = tcdzy
      fadd Dword Ptr[nytc]    'nytc+nxts
      fiadd Dword Ptr[sh2]    'nxts+nytc+sh2
      'fpu stack returns to: st(0) = tsdzx, st(1) = tcdzx, st(2) = tsdzy, st(3) = tcdzy 
      
      mov ebx, [xput]
      add ebx, [dstptr]
      
      mov ecx, Dword Ptr [x_draw_len]
      
      Test ecx, ecx ' 0?
      jz x_end      ' nothing to do here
      
      x_inner:
      
      fist Dword Ptr [my] ' my = _my
      
      fld st(1)           ' mx = _mx
      fistp Dword Ptr [mx]
      
      mov esi, Dword Ptr [mx]         ' esi = mx
      mov edi, Dword Ptr [my]         ' edi = my
      
      ' bounds checking
      Test esi, esi       'mx<0?
      js no_draw          
      'mov esi, 0
      
      Test edi, edi
      'mov edi, 0
      js no_draw          'my<0?
      
      cmp esi, Dword Ptr [srcwidth]   ' mx >= width?
      jge no_draw
      cmp edi, Dword Ptr [srcheight]  ' my >= height?
      jge no_draw
      
      ' calculate position in src buffer
      mov eax, Dword Ptr [srcbyteptr] ' eax = srcbyteptr
      imul edi, Dword Ptr [srcpitch]  ' edi = my * srcpitch
      add eax, edi
      Shl esi, 2
      ' eax becomes src pixel color
      mov eax, Dword Ptr [eax+esi]
      cmp eax, [transcol]
      je no_draw
      
      ' draw pixel
      mov Dword Ptr [ebx], eax
      no_draw:
      
      fld st(3)
      faddp st(2), st(0) ' _mx += tcdzx
      fadd st(0), st(2) ' _my += tsdzx
      
      ' increment the output pointer
      add ebx, 4
      
      ' increment the x loop
      dec ecx
      jnz x_inner
      
      x_end:
      
      fstp Dword Ptr [_my]
      fstp Dword Ptr [_mx]
      
      'st(0) = tsdzx, st(1) = tcdzx, st(2) = tsdzy, st(3) = tcdzy
      'nytc += tcdzy
      fld Dword Ptr[nytc]
      fadd st(0), st(4)
      fstp Dword Ptr[nytc]
      
      'st(0) = tsdzx, st(1) = tcdzx, st(2) = tsdzy, st(3) = tcdzy
      'nyts+=tsdzy
      fld Dword Ptr[nyts]
      fadd st(0), st(3) 
      fstp Dword Ptr[nyts]
      
      'dstptr += dst->pitch
      mov eax, Dword Ptr [dstpitch]
      add Dword Ptr [dstptr], eax
      
      dec edx
      jnz y_inner
      
      y_end:
      
      finit
   
   End Asm
    
End Sub

Function DuplicateImage (img As Any Ptr) As Any Ptr
Dim As FB.IMAGE Ptr fbimg = img
   Return ImageCreate(fbimg->Width,fbimg->height)
End Function
